'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _ramda = require('ramda');

var _ramdaAdjunct = require('ramda-adjunct');

var _breakpointMapping = require('../objects/breakpointMapping');

var _breakpointMap = require('../utils/breakpointMap');

var _breakpoints = require('../utils/breakpoints');

var _functions = require('../utils/functions');

var _list = require('../utils/list');

var _objects = require('../utils/objects');

var _queryDescriptor = require('../utils/queryDescriptor');

// -----------------------------------------------------------------------------
// Process Range
// -----------------------------------------------------------------------------

var addBreakpointValueToRange = function addBreakpointValueToRange(breakpointMap) {
  return (0, _ramda.converge)(_breakpointMapping.assocValue, [(0, _ramda.pipe)(_breakpointMapping.propName, (0, _breakpointMap.findBreakpointByName)(breakpointMap)), _ramda.identity]);
};

var addBreakpointValuesToRange = function addBreakpointValuesToRange(breakpointMap) {
  return (0, _ramda.map)(addBreakpointValueToRange(breakpointMap));
};

var processRange = function processRange(breakpointMap) {
  return (0, _ramda.pipe)(addBreakpointValuesToRange(breakpointMap), (0, _queryDescriptor.calculateQueryDescriptorForRange)(breakpointMap));
};

var parseBreakpointToMap = function parseBreakpointToMap(breakpoints, _ref) {
  var _ref2 = _slicedToArray(_ref, 2),
      name = _ref2[0],
      value = _ref2[1];

  return (0, _ramda.pipe)(_breakpoints.parseBreakpoint, (0, _breakpointMapping.assocValue)(value), (0, _ramdaAdjunct.appendFlipped)(breakpoints))(name);
};

var parseBreakpointsToMap = (0, _objects.reduceObjIndexed)(parseBreakpointToMap, []);

var createMappingByName = function createMappingByName(breakpointMap) {
  return function (mappings, _ref3) {
    var name = _ref3.name,
        range = _ref3.range,
        value = _ref3.value;
    return (0, _ramda.pipe)(processRange(breakpointMap), (0, _breakpointMapping.createBreakpointMapping)(name, value), (0, _ramdaAdjunct.appendFlipped)(mappings))(range);
  };
};

var createMappingsByName = function createMappingsByName(breakpointMap) {
  return (0, _ramda.reduce)(createMappingByName(breakpointMap), []);
};

var createMappingByIndex = function createMappingByIndex(breakpointMap) {
  return function (mappings, value, idx) {
    return (0, _ramda.pipe)((0, _breakpointMap.findBreakpointByIndex)(breakpointMap), (0, _ramda.insert)(1, value), (0, _ramda.apply)(_breakpointMapping.createBreakpointMapping), (0, _ramdaAdjunct.appendFlipped)(mappings))(idx);
  };
};

var createMappingsByIndex = function createMappingsByIndex(breakpointMap) {
  return function (values) {
    return (0, _ramdaAdjunct.reduceIndexed)(createMappingByIndex(breakpointMap), [], values);
  };
};

var addQueryDescriptor = function addQueryDescriptor(mappings) {
  return function (value, idx) {
    return (0, _ramda.pipe)(_queryDescriptor.calculateQueryDescriptor, (0, _breakpointMapping.assocQuery)(_ramda.__, value))(idx, mappings);
  };
};

var addQueryDescriptors = function addQueryDescriptors(mappings) {
  return (0, _ramdaAdjunct.mapIndexed)(addQueryDescriptor(mappings), mappings);
};

// -----------------------------------------------------------------------------
// API
// -----------------------------------------------------------------------------

var createApi = function createApi(breakpointMap) {
  // Resolve breakpoints for values declared using an object syntax
  var byName = (0, _ramda.pipe)(parseBreakpointsToMap, createMappingsByName(breakpointMap));

  // Resolve breakpoints for values declared using an array syntax
  var byIndex = (0, _ramda.pipe)(createMappingsByIndex(breakpointMap), addQueryDescriptors);

  var count = (0, _ramda.once)(function () {
    return (0, _list.numKeys)(breakpointMap);
  });

  return {
    byIndex: byIndex,
    byName: byName,
    count: count
  };
};

// -----------------------------------------------------------------------------
// Exports
// -----------------------------------------------------------------------------

var breakpointProvider = (0, _ramda.pipe)(_functions.defaultToObj, _breakpoints.breakpointValuesToEms, createApi);

exports.default = breakpointProvider;