'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _ramda = require('ramda');

var _ramdaAdjunct = require('ramda-adjunct');

var _errors = require('../errors');

var _scope = require('../objects/scope');

var argIsScopeObj = (0, _ramda.compose)(_scope.hasScope, _ramda.head);

var argIsObj = (0, _ramda.compose)(_ramdaAdjunct.isPlainObject, _ramda.head);

var expandScope = function expandScope(provider) {
  return function (value) {
    return (0, _ramda.times)((0, _ramda.always)(value), provider.count());
  };
};

var resolveBreakpointsImpl = function resolveBreakpointsImpl(provider) {
  return function (breakpoint) {
    return (0, _ramda.cond)([[argIsScopeObj, (0, _ramda.pipe)(_ramda.head, _scope.propScope, expandScope(provider), resolveBreakpointsImpl(provider)
    // run through and remove values that don't change.
    )], [argIsObj, (0, _ramda.compose)(provider.byName, _ramda.head)], [_ramda.T, provider.byIndex]])(breakpoint);
  };
};

var resolveBreakpoints = function resolveBreakpoints(provider) {
  return function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return (0, _ramda.tryCatch)(resolveBreakpointsImpl(provider), function (message) {
      return (0, _errors.throwBreakpointError)((0, _errors.invalidBreakpointError)(message, args));
    })(args);
  };
};

exports.default = resolveBreakpoints;