'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _ramda = require('ramda');

var _ramdaAdjunct = require('ramda-adjunct');

var _breakpointMapping = require('../../objects/breakpointMapping');

var _transformers = require('../../utils/transformers');

var _renderSingleDeclaration = require('./renderers/renderSingleDeclaration');

var _renderSingleDeclaration2 = _interopRequireDefault(_renderSingleDeclaration);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var renderDeclaration = function renderDeclaration(propName, renderer) {
  return (0, _ramda.pipe)(_ramdaAdjunct.ensureArray, (0, _ramda.partial)(renderer, [propName]), _ramdaAdjunct.list);
};

var processDeclaration = function processDeclaration(propName, data, _ref, _ref2) {
  var _ref$transformers = _ref.transformers,
      transformers = _ref$transformers === undefined ? _ramda.identity : _ref$transformers,
      _ref$renderer = _ref.renderer,
      renderer = _ref$renderer === undefined ? _renderSingleDeclaration2.default : _ref$renderer;
  var name = _ref2.name,
      query = _ref2.query,
      value = _ref2.value;
  return (0, _ramda.pipe)((0, _transformers.transformDeclarationValue)(transformers, name, data), renderDeclaration(propName, renderer), (0, _breakpointMapping.createBreakpointMapping)(name, _ramda.__, query))(value);
};

// Create a map of curried 'processDeclaration' functions for each property,
// ready to render styles when they receive a value
var reducer = function reducer(propName, data, style) {
  return function (declarations, breakpointMapping) {
    return (0, _ramda.pipe)(processDeclaration, (0, _ramdaAdjunct.appendFlipped)(declarations))(propName, data, style, breakpointMapping);
  };
};

var createDeclarationProcessor = function createDeclarationProcessor(propName, data, style) {
  return (0, _ramda.reduce)(reducer(propName, data, style), []);
};

exports.default = createDeclarationProcessor;