'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _ramda = require('ramda');

var _ramdaAdjunct = require('ramda-adjunct');

var _config = require('../../const/config');

var _config2 = _interopRequireDefault(_config);

var _errors = require('../../errors');

var _config3 = require('../../objects/config');

var _formatting = require('../../utils/formatting');

var _list = require('../../utils/list');

var _logic = require('../../utils/logic');

var _predicate = require('../../utils/predicate');

var _transformers = require('../../utils/transformers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var SCOPES = _config2.default.SCOPES,
    ALIASES = _config2.default.ALIASES;


var validPrefixes = function validPrefixes(config, data) {
  return (0, _ramda.pipe)(_ramda.keys, (0, _ramda.without)([SCOPES, ALIASES]), (0, _ramdaAdjunct.concatRight)((0, _ramda.keys)(config.data[ALIASES])))(data);
};

var expandData = function expandData(config) {
  // Build Function to resolve aliases
  var resolveDataAlias = function resolveDataAlias(name, data) {
    return (0, _ramda.pipe)(_ramda.prop, (0, _logic.whenIsUndefined)(function () {
      return (0, _ramda.prop)(name, config.data.aliases);
    }), (0, _logic.whenIsUndefined)(function () {
      (0, _errors.throwDataError)((0, _errors.unrecognisedDataPrefixError)(name, validPrefixes(config, data)));
    }))(name, data);
  };

  // ---------------------------------------------------------------------------
  // Value Expansion
  // ---------------------------------------------------------------------------

  var resolveName = function resolveName(sourceData, name, key) {
    return (0, _ramda.pipe)((0, _ramda.prop)(name), (0, _logic.whenIsUndefined)(function () {
      return (0, _errors.throwDataError)((0, _errors.missingDataNodeError)(name));
    }), (0, _ramda.prop)(key), (0, _logic.whenIsUndefined)(function () {
      return (0, _errors.throwDataError)((0, _errors.missingDataItemKeyError)(name, key));
    }))(sourceData);
  };

  var expandToken = function expandToken(sourceData) {
    return function (token) {
      var _splitOnColon = (0, _formatting.splitOnColon)(token),
          _splitOnColon2 = _slicedToArray(_splitOnColon, 2),
          prefix = _splitOnColon2[0],
          key = _splitOnColon2[1];

      var name = resolveDataAlias(prefix, sourceData);
      var value = resolveName(sourceData, name, key);
      // eslint-disable-next-line no-use-before-define
      return expandValue(sourceData)(value);
    };
  };

  var expandValues = function expandValues(sourceData) {
    return (
      // eslint-disable-next-line no-use-before-define
      (0, _ramda.map)(expandValue(sourceData))
    );
  };

  var expandValue = function expandValue(sourceData) {
    return function (v) {
      return (0, _ramda.cond)([[_predicate.isToken, expandToken(sourceData)], [_predicate.isCSSFunction, (0, _transformers.transformFunctionElements)(expandValues(sourceData))], [_predicate.hasUnnestedWhitespace, (0, _transformers.transformGroup)(expandValue(sourceData))], [_ramda.T, _ramda.identity]])(v);
    };
  };

  // ---------------------------------------------------------------------------
  // Node Expansion
  // ---------------------------------------------------------------------------

  var expandNode = function expandNode(name, expandedRootData) {
    return (0, _list.reduceWithKeys)(function (node, key) {
      var sourceData = (0, _ramda.over)((0, _ramda.lensProp)(name), (0, _ramda.mergeDeepLeft)(node), expandedRootData);
      return (0, _ramda.over)((0, _ramda.lensProp)(key), (0, _ramda.when)(_ramdaAdjunct.isString, expandValue(sourceData)))(node);
    });
  };

  var expandNodes = function expandNodes(expandedRootData) {
    return (0, _list.reduceWithKeys)(function (acc, name) {
      return (0, _ramda.pipe)(expandNode, (0, _ramda.over)((0, _ramda.lensProp)(name), _ramda.__, acc))(name, expandedRootData);
    });
  };

  // ---------------------------------------------------------------------------
  // Scopes
  // ---------------------------------------------------------------------------

  var expandScopes = function expandScopes(expandedRootData) {
    return (0, _ramda.map)((0, _ramda.over)(_config3.lData, expandNodes(expandedRootData)));
  };

  // ---------------------------------------------------------------------------
  // Data
  // ---------------------------------------------------------------------------

  var expand = function expand(data) {
    // Expand the root node using itself as a data source
    var expandedRootData = expandNodes(data)(data);

    // Expand scoped nodes
    return (0, _ramda.pipe)(_config3.propScopes, (0, _ramda.when)(_ramdaAdjunct.isNotUndefined, expandScopes(expandedRootData)), (0, _ramda.assoc)(SCOPES, _ramda.__, expandedRootData))(data);
  };

  return (0, _ramda.over)(_config3.lData, expand, config);
};

exports.default = expandData;