'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _ramda = require('ramda');

var _ramdaAdjunct = require('ramda-adjunct');

var _breakpoints = require('../const/breakpoints');

var _errors = require('../errors');

var _config = require('../objects/config');

var _logic = require('../utils/logic');

var _predicate = require('../utils/predicate');

var filterResolvingScopes = function filterResolvingScopes(breakpointName) {
  return (0, _ramda.filter)((0, _ramdaAdjunct.lensSatisfies)((0, _ramda.contains)(breakpointName), _config.lResolve));
};

// Find any scopes that resolve to this breakpoint, reversing them so that the
// final scope is first, adding the default scope in last position.
var findResolvingScopes = function findResolvingScopes(breakpointName, data) {
  return (0, _ramda.pipe)(_config.propScopes, filterResolvingScopes(breakpointName), (0, _ramda.map)(_config.propData), _ramda.reverse, (0, _ramda.append)(data))(data);
};

// Merge any props that don't already exist on the object
var resolveObject = function resolveObject(acc) {
  return (0, _ramda.when)((0, _ramda.always)((0, _ramdaAdjunct.isObject)(acc)), (0, _ramdaAdjunct.mergeRight)(acc));
};

var findPropOnScope = function findPropOnScope(dataPropName) {
  return function (acc, scopeData) {
    return (0, _ramda.pipe)((0, _ramda.prop)(dataPropName), (0, _ramda.cond)([[_ramda.isNil, (0, _ramda.always)(acc)], [_ramdaAdjunct.isObject, resolveObject(acc)], [_ramda.T, _ramda.reduced]]))(scopeData);
  };
};

var findPropsOnScope = function findPropsOnScope(dataPropName) {
  return (0, _ramda.reduce)(findPropOnScope(dataPropName), null);
};

// Find any matching prop on this dataNode
var resolveScopes = function resolveScopes(data, dataPropName) {
  return function (breakpointName) {
    return (0, _ramda.pipe)(findResolvingScopes, findPropsOnScope(dataPropName))(breakpointName, data);
  };
};

var shouldResolveScopes = function shouldResolveScopes(data) {
  return (0, _ramda.either)(_predicate.isNotDefaultBreakpoint, function () {
    return (0, _predicate.hasScopes)(data);
  });
};

var findDataProp = function findDataProp(breakpointName, dataPropName) {
  return function (data) {
    return (0, _ramda.pipe)(
    // Try using scopes
    (0, _ramda.when)(shouldResolveScopes(data), resolveScopes(data, dataPropName)),
    // Fall back to root
    (0, _ramda.defaultTo)((0, _ramda.prop)(dataPropName, data)))(breakpointName);
  };
};

var resolveKeyToValue = function resolveKeyToValue(dataPropName) {
  return function (value, data) {
    var breakpointName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _breakpoints.DEFAULT_BREAKPOINT_NAME;
    return (0, _ramda.pipe)(findDataProp(breakpointName, dataPropName), (0, _logic.whenIsUndefined)(function () {
      return (0, _errors.throwDataError)((0, _errors.missingDataNodeError)(dataPropName));
    }))(data);
  };
};

exports.default = resolveKeyToValue;