'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.breakpointValuesToEms = exports.parseBreakpoint = exports.addDefaultBreakpoint = undefined;

var _ramda = require('ramda');

var _ramdaAdjunct = require('ramda-adjunct');

var _breakpoints = require('../const/breakpoints');

var _errors = require('../errors');

var _breakpointDescriptor = require('../objects/breakpointDescriptor');

var _breakpointDescriptor2 = _interopRequireDefault(_breakpointDescriptor);

var _rootPxToEmTransformer = require('../transformers/rootPxToEmTransformer');

var _rootPxToEmTransformer2 = _interopRequireDefault(_rootPxToEmTransformer);

var _predicate = require('./predicate');

var _range = require('./range');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var addDefaultBreakpoint = exports.addDefaultBreakpoint = (0, _ramda.prepend)([_breakpoints.DEFAULT_BREAKPOINT_NAME, 0]);

var parseBreakpoint = exports.parseBreakpoint = function parseBreakpoint(value) {
  if (!(0, _predicate.isValidModifiedMq)(value)) {
    throw Error((0, _errors.invalidBreakpointSyntaxError)(value));
  }

  // A range will always have a < in a position greater than 0
  if (!(0, _predicate.isRange)(value)) {
    // It's a single item
    var rangeItem = (0, _range.createRangeItem)(value);
    return (0, _breakpointDescriptor2.default)(value, rangeItem);
  }

  // Otherwise it's a range (containing two items)
  return (0, _ramda.pipe)((0, _ramda.split)(_breakpoints.LT_MODIFIER), (0, _ramda.map)(_range.createRangeItem), (0, _breakpointDescriptor2.default)(value))(value);
};

var breakpointValuesToEms = exports.breakpointValuesToEms = (0, _ramda.map)((0, _ramda.apply)((0, _ramda.useWith)(_ramdaAdjunct.list, [_ramda.identity, (0, _ramda.unless)(_predicate.isEmString, _rootPxToEmTransformer2.default)])));