'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.optimiseDeclarations = exports.addBreakpointToDeclarations = exports.batchDeclarations = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _ramda = require('ramda');

var _ramdaAdjunct = require('ramda-adjunct');

var _breakpointMapping = require('../const/breakpointMapping');

var _breakpointMapping2 = _interopRequireDefault(_breakpointMapping);

var _queryDescriptor = require('../const/queryDescriptor');

var _queryDescriptor2 = _interopRequireDefault(_queryDescriptor);

var _errors = require('../errors');

var _breakpointMapping3 = require('../objects/breakpointMapping');

var _objects = require('../utils/objects');

var _predicate = require('../utils/predicate');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var QUERY = _breakpointMapping2.default.QUERY;
var TO = _queryDescriptor2.default.TO;


var foundMatch = (0, _ramda.flip)(_ramda.gte)(0);

var findBatchIndexForMapping = function findBatchIndexForMapping(_ref, batches) {
  var name = _ref.name,
      query = _ref.query;
  return (0, _ramda.findIndex)((0, _ramda.both)((0, _ramdaAdjunct.lensEq)(_breakpointMapping3.lName, name), (0, _ramdaAdjunct.lensEq)(_breakpointMapping3.lQuery, query)), batches);
};

var createNewBatch = function createNewBatch(breakpointMapping, batches) {
  return (0, _ramda.append)(breakpointMapping, batches);
};

var addToBatch = function addToBatch(_ref2) {
  var name = _ref2.name,
      query = _ref2.query,
      value = _ref2.value;
  return (0, _ramda.pipe)(_breakpointMapping3.propValue, (0, _ramdaAdjunct.concatRight)(value), (0, _breakpointMapping3.createBreakpointMapping)(name, _ramda.__, query));
};

var addToBatchAtIndex = (0, _ramda.curry)(function (breakpointMapping, batches, index) {
  return (0, _ramda.over)((0, _ramda.lensIndex)(index), addToBatch(breakpointMapping), batches);
});

var batchDeclarations = exports.batchDeclarations = (0, _ramda.reduce)(function (batches, breakpointMapping) {
  return (0, _ramda.pipe)(findBatchIndexForMapping, (0, _ramda.ifElse)(foundMatch, addToBatchAtIndex(breakpointMapping, batches), function () {
    return createNewBatch(breakpointMapping, batches);
  }))(breakpointMapping, batches);
}, []);

var addBreakpoingToDeclaration = function addBreakpoingToDeclaration(breakpoint) {
  return function (acc, _ref3) {
    var _ref4 = _slicedToArray(_ref3, 2),
        name = _ref4[0],
        value = _ref4[1];

    return (0, _ramda.pipe)((0, _ramda.unless)(_predicate.isValidMqValue, function () {
      return (0, _errors.throwMQError)((0, _errors.unsupportedBreakpointValuesError)(value));
    }), (0, _ramda.objOf)(breakpoint), (0, _ramda.assoc)(name, _ramda.__, acc))(value);
  };
};

var addBreakpointToDeclarations = exports.addBreakpointToDeclarations = function addBreakpointToDeclarations(breakpoint, declarations) {
  return (0, _objects.reduceObjIndexed)(addBreakpoingToDeclaration(breakpoint), {}, declarations);
};

var assocPathQueryTo = (0, _ramda.assocPath)([QUERY, TO]);

var compareMappings = function compareMappings(previous, current) {
  return current.value !== '_' && (!previous || !(0, _ramda.equals)(previous.value, current.value));
};

var replaceToValue = function replaceToValue(newToValue) {
  return (0, _ramda.ifElse)((0, _ramda.always)((0, _ramdaAdjunct.isNotNil)(newToValue)), assocPathQueryTo(newToValue), (0, _ramda.over)(_breakpointMapping3.lQuery, (0, _ramda.omit)([TO])));
};

var updateQuery = function updateQuery(newToValue) {
  return (0, _ramda.pipe)(_ramda.last, replaceToValue(newToValue));
};

var updateLastMapping = function updateLastMapping(acc, newToValue) {
  return (0, _ramda.converge)(_ramda.append, [updateQuery(newToValue), _ramda.init])(acc);
};

// This is gross
var optimiseDeclarations = exports.optimiseDeclarations = function optimiseDeclarations(mappings) {
  var lastMapping = void 0;
  return (0, _ramda.reduce)(function (acc, mapping) {
    var result = void 0;
    var shouldAdd = compareMappings(lastMapping, mapping);
    if (shouldAdd) {
      lastMapping = mapping;
      result = (0, _ramda.append)(mapping, acc);
    } else {
      result = updateLastMapping(acc, mapping.query.to);
      lastMapping = (0, _ramda.last)(result);
    }
    return result;
  }, [], mappings);
};