'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addEmValues = exports.tokenName = exports.unitPartOfUnitedNumberString = exports.numericPartOfUnitedNumberString = exports.elementsOfUnitedNumberString = undefined;

var _ramda = require('ramda');

var _errors = require('../errors');

var _formatting = require('./formatting');

var _numbers = require('./numbers');

var elementsOfUnitedNumberString = exports.elementsOfUnitedNumberString = function elementsOfUnitedNumberString(value) {
  var captures = /^(-?\d+(?:.\d+)?)([a-z|%]+)?$/.exec(value);
  if (!captures || (0, _ramda.any)(_ramda.isNil, [captures, captures[1], captures[2]])) {
    (0, _errors.throwParseError)((0, _errors.unitedNumberError)(value));
  }
  return [Number(captures[1]), captures[2]];
};

var numericPartOfUnitedNumberString = exports.numericPartOfUnitedNumberString = (0, _ramda.compose)((0, _ramda.view)((0, _ramda.lensIndex)(0)), elementsOfUnitedNumberString);

var unitPartOfUnitedNumberString = exports.unitPartOfUnitedNumberString = (0, _ramda.compose)((0, _ramda.view)((0, _ramda.lensIndex)(1)), elementsOfUnitedNumberString);

var tokenName = exports.tokenName = (0, _ramda.pipe)(_formatting.splitOnColon, _ramda.last);

var addEmValues = exports.addEmValues = (0, _ramda.curry)(function (emValue1, emValue2) {
  var value1 = (0, _ramda.unless)((0, _ramda.equals)(0), numericPartOfUnitedNumberString)(emValue1);
  var value2 = (0, _ramda.unless)((0, _ramda.equals)(0), numericPartOfUnitedNumberString)(emValue2);
  var result = (0, _numbers.clampPositive)(value1 + value2);
  return (0, _ramda.when)((0, _ramda.gt)(_ramda.__, 0), function (v) {
    return v + 'em';
  }, result);
});