'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.applyOffsetToBreakpointValue = exports.applyOffset = exports.hasNoOffset = exports.createRangeItem = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _ramda = require('ramda');

var _breakpoints = require('../const/breakpoints');

var _breakpointMapping = require('../objects/breakpointMapping');

var _rangeItem = require('../objects/rangeItem');

var _rootPxToEmTransformer = require('../transformers/rootPxToEmTransformer');

var _rootPxToEmTransformer2 = _interopRequireDefault(_rootPxToEmTransformer);

var _parse = require('./parse');

var _predicate = require('./predicate');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var extractPositiveOffset = (0, _ramda.split)(_breakpoints.POSITIVE_OFFSET);

var extractNegativeOffset = function extractNegativeOffset(v) {
  var _split = (0, _ramda.split)(_breakpoints.NEGATIVE_OFFSET, v),
      _split2 = _slicedToArray(_split, 2),
      name = _split2[0],
      offset = _split2[1];

  return [name, '-' + offset];
};

var extractOffset = (0, _ramda.cond)([[_predicate.hasPositiveOffset, extractPositiveOffset], [_predicate.hasNegativeOffset, extractNegativeOffset], [_ramda.T, _ramda.of]]);

var firstCharIsModifier = function firstCharIsModifier(value) {
  return (0, _ramda.any)((0, _ramda.equals)(value[0]), _breakpoints.MODIFIERS);
};

// -----------------------------------------------------------------------------
// Exports
// -----------------------------------------------------------------------------

var createRangeItem = exports.createRangeItem = function createRangeItem(value) {
  var modifier = firstCharIsModifier(value) ? (0, _ramda.head)(value) : null;
  if (modifier) value = (0, _ramda.tail)(value);

  var _extractOffset = extractOffset(value),
      _extractOffset2 = _slicedToArray(_extractOffset, 2),
      name = _extractOffset2[0],
      offset = _extractOffset2[1];

  var result = {
    name: name,
    offset: offset,
    modifier: modifier
  };

  return (0, _ramda.reject)(_ramda.isNil, result);
};

var hasNoOffset = exports.hasNoOffset = (0, _ramda.pipe)(_rangeItem.propOffset, _ramda.isNil);

var applyOffset = exports.applyOffset = function applyOffset(rangeItem) {
  return (0, _ramda.pipe)(_rangeItem.propOffset, (0, _ramda.unless)(_predicate.isEmString, _rootPxToEmTransformer2.default), (0, _parse.addEmValues)(rangeItem.value));
};

var applyOffsetToBreakpointValue = exports.applyOffsetToBreakpointValue = function applyOffsetToBreakpointValue(rangeItem) {
  return (0, _ramda.cond)([[hasNoOffset, _breakpointMapping.propValue], [_ramda.T, applyOffset(rangeItem)]])(rangeItem);
};