'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformDeclarationValue = exports.transformGroup = exports.transformValueAt = exports.transformValues = exports.transformValue = exports.transformCalcElements = exports.transformFunctionElements = exports.prepareForTransform = undefined;

var _ramda = require('ramda');

var _ramdaAdjunct = require('ramda-adjunct');

var _regexp = require('../const/regexp');

var _formatting = require('./formatting');

var _predicate = require('./predicate');

var _templates = require('./templates');

var prepareForTransform = exports.prepareForTransform = (0, _ramda.pipe)((0, _ramda.when)(_predicate.isNotStringOrArray, String), (0, _ramda.when)(_ramdaAdjunct.isString, _formatting.splitOnUnnestedWhitespace));

// -----------------------------------------------------------------------------
// CSS Function name()
// -----------------------------------------------------------------------------

var transformArgumentPart = function transformArgumentPart(transformers) {
  return (0, _ramda.pipe)(_ramda.trim, _formatting.splitOnWhitespace, (0, _ramda.map)((0, _ramda.pipe)(_ramdaAdjunct.ensureArray, transformers)), _formatting.joinWithSpace);
};

var transformArgumentParts = function transformArgumentParts(transformers) {
  return (0, _ramda.map)(transformArgumentPart(transformers));
};

var transformFunctionArguments = function transformFunctionArguments(transformers) {
  return (0, _ramda.pipe)(_formatting.splitOnUnnestedComma, transformArgumentParts(transformers), _formatting.joinWithCommaSpace);
};

var typeOfFunction = (0, _ramda.pipe)((0, _ramda.match)(_regexp.RE_CSS_FUNCTION_NAME), (0, _ramda.nth)(1));

var transformFunctionElements = exports.transformFunctionElements = (0, _ramda.curry)(function (transformers, value) {
  return (0, _ramda.pipe)(_ramda.trim, _formatting.extractFunctionArguments, transformFunctionArguments(transformers), (0, _templates.createCSSFunctionFromTemplate)(typeOfFunction(value)))(value);
});

var transformCalcElements = exports.transformCalcElements = function transformCalcElements(transformers) {
  return (0, _ramda.pipe)(_ramda.trim, _formatting.extractFunctionArguments, (0, _ramda.replace)(_regexp.RE_CALC_VALUES, transformers), (0, _templates.createCSSFunctionFromTemplate)('calc'));
};

// -----------------------------------------------------------------------------
// Transform Basic Value - after we have parsed to smallest possible value
// -----------------------------------------------------------------------------

var prepareTransformers = (0, _ramda.pipe)(_ramdaAdjunct.ensureArray, _ramda.flatten);

var applyTransformers = function applyTransformers(value, data, breakpointName) {
  return (0, _ramda.reduce)(function (currentValue, transformer) {
    return transformer(currentValue, data, breakpointName);
  }, value);
};

var transformValue = exports.transformValue = (0, _ramda.curry)(function (transformers, data, breakpointName, value) {
  return (0, _ramda.pipe)(prepareTransformers, applyTransformers(value, data, breakpointName))(transformers);
});

var transformValues = exports.transformValues = (0, _ramda.curry)(function (transformers, data, breakpointName, values) {
  return (0, _ramda.pipe)(prepareForTransform, (0, _ramda.map)(transformValue(transformers, data, breakpointName)))(values);
});

var transformValueAt = exports.transformValueAt = (0, _ramda.curry)(function (transformers, data, breakpointName, position, value) {
  return (0, _ramda.pipe)(prepareForTransform, (0, _ramda.adjust)(transformValues(transformers, data, breakpointName), position))(value);
});

// -----------------------------------------------------------------------------
// Transform Group (values, separated, with, comma)
// -----------------------------------------------------------------------------

var transformGroupMember = function transformGroupMember(transformers, data, breakpointName) {
  return (0, _ramda.pipe)(_formatting.splitOnUnnestedWhitespace, transformValues(transformers, data, breakpointName), _formatting.joinWithSpace);
};

var transformGroupMembers = function transformGroupMembers(transformers, data, breakpointName) {
  return (0, _ramda.map)(transformGroupMember(transformers, data, breakpointName));
};

var transformGroup = exports.transformGroup = function transformGroup(transformers, data, breakpointName) {
  return (0, _ramda.pipe)(_formatting.splitOnUnnestedComma, _formatting.trimAll, transformGroupMembers(transformers, data, breakpointName), _formatting.joinWithCommaSpace);
};

var transformDeclarationValue = exports.transformDeclarationValue = function transformDeclarationValue(transformers, breakpointName, data) {
  return (0, _ramda.ifElse)(_predicate.isGroupString, transformGroup(transformers, data, breakpointName), transformValue(transformers, data, breakpointName));
};